#!/usr/bin/python -u

"""
Copyright (C) 2017 Jacksgong(blog.dreamtobe.cn)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import argparse
from sys import argv

from helper import LOG_LEVELS, is_path
from logfile_parser import LogFileParser

from okcat.adb import Adb
from okcat.terminalcolor import print_tips, print_blue

__author__ = 'JacksGong'
__version__ = '1.0.6'
__description__ = 'This python script used for combine several Android projects to one project.'


def main():
    print("-------------------------------------------------------")
    print("                  OkCat v" + __version__)
    print ""
    print "Thanks for using okcat! Now, the doc is available on: "
    print_blue("        https://github.com/Jacksgong/okcat")
    print ""
    print "                   Have Fun!"
    print("-------------------------------------------------------")

    parser = argparse.ArgumentParser(description='Filter logcat by package name')
    parser.add_argument('package_or_path', nargs='*',
                        help='This can be Application package name(s) or log file path(if the file from path is exist)')
    parser.add_argument('-y', '--yml_file_name', dest='yml', help='Using yml file you config on ~/.okcat folder')
    parser.add_argument('--hide-same-tags', dest='hide_same_tags', action='store_true',
                        help='Do not display the same tag name')

    # following args are just for parser
    # parser.add_argument('-k', '--keyword', dest='keyword', action='append', help='You can filter you care about log by this keyword(s)')

    # following args are just for adb
    parser.add_argument('-w', '--tag-width', metavar='N', dest='tag_width', type=int, default=23,
                        help='Width of log tag')
    parser.add_argument('-l', '--min-level', dest='min_level', type=str, choices=LOG_LEVELS + LOG_LEVELS.lower(),
                        default='V', help='Minimum level to be displayed')
    parser.add_argument('--color-gc', dest='color_gc', action='store_true', help='Color garbage collection')
    parser.add_argument('--current', dest='current_app', action='store_true',
                        help='Filter logcat by current running app')
    parser.add_argument('-s', '--serial', dest='device_serial', help='Device serial number (adb -s option)')
    parser.add_argument('-d', '--device', dest='use_device', action='store_true',
                        help='Use first device for log input (adb -d option)')
    parser.add_argument('-e', '--emulator', dest='use_emulator', action='store_true',
                        help='Use first emulator for log input (adb -e option)')
    parser.add_argument('-c', '--clear', dest='clear_logcat', action='store_true',
                        help='Clear the entire log before running')
    parser.add_argument('-t', '--tag', dest='tag', action='append', help='Filter output by specified tag(s)')
    parser.add_argument('-tk', '--tag_keywords', dest='tag_keywords', action='append',
                        help='Filter output by specified tag keyword(s)')
    parser.add_argument('-i', '--ignore-tag', dest='ignored_tag', action='append',
                        help='Filter output by ignoring specified tag(s)')
    parser.add_argument('-a', '--all', dest='all', action='store_true', default=False,
                        help='Print all log messages')

    # help
    if len(argv) == 2 and argv[1] == 'help':
        exit()

    args = parser.parse_args()

    file_path = None
    candidate_path = args.package_or_path
    if candidate_path is not None and len(candidate_path) > 0 and is_path(candidate_path[0]):
        file_path = candidate_path[0]

    if file_path is None:
        _adb = Adb()
        _adb.setup(args)
        while True:
            try:
                _adb.loop()
            except KeyboardInterrupt:
                break

    else:
        parser = LogFileParser(file_path, args.hide_same_tags)
        parser.setup(args.yml)
        print u''.join(parser.parse()).encode('utf-8').strip()



#!/usr/bin/python -u

"""
Copyright (C) 2017 Jacksgong(blog.dreamtobe.cn)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from os.path import exists

from helper import get_conf_path
from logprocessor import LogProcessor
from terminalcolor import colorize, allocate_color

from okcat.confloader import ConfLoader


class LogFileParser:
    file_path = None
    valid = False
    processor = None
    hide_same_tags = None

    def __init__(self, file_path, hide_same_tags):
        self.file_path = file_path
        self.hide_same_tags = hide_same_tags

    def setup(self, yml_file_name):
        if not exists(self.file_path):
            exit("log path: %s is not exist!" % self.file_path)
        self.processor = LogProcessor(self.hide_same_tags)

        loader = ConfLoader()
        loader.load(get_conf_path(yml_file_name))

        self.processor.setup_trans(trans_msg_map=loader.get_trans_msg_map(),
                                   trans_tag_map=loader.get_trans_tag_map(),
                                   hide_msg_list=loader.get_hide_msg_list())
        self.processor.setup_separator(separator_rex_list=loader.get_separator_regex_list())
        self.processor.setup_highlight(highlight_list=loader.get_highlight_list())
        self.processor.setup_condition(tag_keywords=loader.get_tag_keyword_list())
        self.processor.setup_regex_parser(regex_exp=loader.get_log_line_regex())

    def parse(self):
        log_file = open(self.file_path, 'r')

        result = ''
        for line in log_file:
            msg_key, linebuf, match_precondition = self.processor.process(line)

            if not match_precondition:
                continue

            if msg_key is not None:
                result += '\n'
                result += colorize(msg_key + ": ", fg=allocate_color(msg_key))
                result += '\n'

            result += linebuf
            result += '\n'

        return result



#!/usr/bin/python -u

"""
Copyright (C) 2017 Jacksgong(blog.dreamtobe.cn)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import re

__author__ = 'JacksGong'


class LogSeparator:
    separator_rex_list = list()
    pre_separate_key = None

    def __init__(self, separator_rex_list):
        for regex_string in separator_rex_list:
            self.separator_rex_list.append(re.compile(r'%s' % regex_string))

    def process(self, msg):
        key = None
        for regex in self.separator_rex_list:
            matched_obj = regex.match(msg)
            if matched_obj is not None:
                key = matched_obj.groups()[0]
                break

        if self.pre_separate_key is None:
            if key is None:
                key = "unknown"
            self.pre_separate_key = key
            return key
        elif key is not None and self.pre_separate_key != key:
            return key
        else:
            return None
